#!/usr/bin/env python

'''
DO NOT EDIT THIS FILE. Make a copy of this file and edit it to fill your needs
while keeping the same program structure.

Template script for testing of controllers for the LBR via FRI while in 
$stiffness.strategy=10, or adding joint torque when $stifness.strategy=30.
Use FRIServer(simulation=True) to connect to a script based on frisim_template,
the default IP address is in this case is 127.0.0.1, port 49938.

Use FRIServer() or FRIServer(simulation=False) to connect to the LBR. 
The default IP address is in this case is 192.168.0.2, port 49938. 
It works using the KRL programs frimeas for position control, and frimeas_trq 
for torque control.
'''

import sys

import numpy as np

from fripy import FRIServer
from fricomm import (FRI_STATE_MON, FRI_STATE_CMD,
                     FRI_CTRL_POSITION, FRI_CTRL_JNT_IMP, LBR_MNJ) 

ZEROS_JNT = [0] * LBR_MNJ

def _ctl_get_setpoints_test():
    # it is safe to delete this function
    maxval = 0.1
    qrise = np.arange(0., maxval, 0.001)
    qdown = np.arange(maxval, 0., -0.001)
    qflat = np.ones(100) * maxval
    qp = np.concatenate((qrise, qflat, qdown))
    qm = -qp * 1.1
    return np.concatenate((qp,qm))

def main():
    fri = FRIServer(simulation=True)
    q0 = _ctl_get_setpoints_test()
    # auxiliary variables
    old_quality = fri.msr.intf.quality
    i = 0
    k = 0
    cmd_mode_once = False

    while True:
        fri.recv_data()

        if (fri.msr.intf.state is FRI_STATE_CMD) and (k < len(q0) - 1):
            cmd_mode_once = True
            k += 1
        
        if (fri.msr.robot.control is FRI_CTRL_POSITION):
            newJntPos = fri.msr.data.cmdJntPos 
            newJntPos[0] += q0[k]
            fri.ctl_jnt_pos(newJntPos)
        elif (fri.msr.robot.control is FRI_CTRL_JNT_IMP):
            addTrq = ZEROS_JNT
            addTrq[0] = q0[k] * 8
            fri.ctl_jnt_imp(newJntTrq=addTrq, 
                            newJntStf=ZEROS_JNT, newJntDmp=ZEROS_JNT)
        
        i += 1
        if (i % 250 is 0):
            fri.print_fri_statistics()
            print(i)
            #print('position', fri.msr.data.msrJntPos)
            #print('torque', fri.cmd.cmd.addJntTrq)
        if (old_quality is not fri.msr.intf.quality):
            fri.print_fri_status()
        old_quality = fri.msr.intf.quality
        
        if (fri.msr.intf.state is FRI_STATE_MON) and (cmd_mode_once is True):
            return 0
        
if __name__ == '__main__':
    status = main()
    sys.exit(status)
