#!/usr/bin/env python

'''
Template script for simulating the position or torque based controllers for the
LBR via FRI using $stiffness.strategy=10 or $stifness.strategy=30, respectively.
It works using the python scripts based on the frictl_template using the 
simulate=True flag. The default IP address is 127.0.0.1, port 49938. 
Make a copy of this file and edit to fill your needs while maintaining the same
structure.
'''

import sys

import numpy as np

from fripy import FRIClient
from fricomm import FRI_STATE_CMD, FRI_CTRL_POSITION, FRI_CTRL_JNT_IMP, LBR_MNJ
from fricomm import FRI_QUALITY_PERFECT 

# Parameters defined by the user
DT = 0.01   # sampling period in seconds
#STRAT = FRI_CTRL_POSITION 
STRAT = FRI_CTRL_JNT_IMP

def _lbrsim_jntpos_compute_next_state(cmd_abs_jnt_pos):
    # safe to delete this function. Input a list, return a list  
    return cmd_abs_jnt_pos
    
def _lbrsim_jntimp_compute_next_state(msr_abs_jnt_pos, add_trq):
    # safe to delete this function. Input lists, return list. Use a dummy model.
    next_abs_jnt_pos = np.array(msr_abs_jnt_pos) + np.array(add_trq) * 0.01
    return list(next_abs_jnt_pos)

def main():
    fri = FRIClient()
    fri.set_control_strategy(STRAT)
    fri.open(DT * 1000)
    while True:
        fri.send_data()
        fri.recv_data()
        if (fri.msr.intf.quality is  FRI_QUALITY_PERFECT and 
            fri.msr.intf.state is not FRI_STATE_CMD):
            fri.start()
        if fri.msr.intf.state is FRI_STATE_CMD:
            if fri.msr.robot.control is FRI_CTRL_POSITION:
                fri.msr.data.msrJntPos = _lbrsim_jntpos_compute_next_state(
                                                        fri.cmd.cmd.jntPos)
            elif fri.msr.robot.control is FRI_CTRL_JNT_IMP:
                fri.msr.data.msrJntPos = _lbrsim_jntimp_compute_next_state(
                                            fri.msr.data.msrJntPos, 
                                            fri.cmd.cmd.addJntTrq)           
        # TODO: a nice way to stop the while loop, or restart simulation
        # return 0
        
if __name__ == '__main__':
    status = main()
    sys.exit(status)
