"""FRI integer constants and data structure definitions. """

LBR_MNJ = 7  # Number of robot joints
FRI_USER_SIZE = 16  # Elements of KRL exchange data
FRI_CART_FRM_DIM = 12  # Elements in homogeneous transformation matrix
FRI_CART_VEC = 6  # Elements in cartesian vector

FRI_DATAGRAM_ID_CMD = 0x1005
FRI_DATAGRAM_ID_MSR = 0x2006

FRI_CMD_JNTPOS = 0x0001
# FRI_CMD_JNTVEL = 0x0002 # not implemented
FRI_CMD_JNTTRQ = 0x0004
FRI_CMD_JNTSTIFF = 0x0010
FRI_CMD_JNTDAMP = 0x0020
FRI_CMD_CARTPOS = 0x0100
# FRI_CMD_CARTVEL = 0x0200 # not implemented
FRI_CMD_TCPFT = 0x0400
FRI_CMD_CARTSTIFF = 0x1000
FRI_CMD_CARTDAMP = 0x2000    


FRI_STATE_INVALID = -1 
FRI_STATE_OFF = 0 
FRI_STATE_MON = 1 
FRI_STATE_CMD = 2 

class FRIMode(object):
    """ Enumeration of FRI states """
    INVALID = 0
    OFF = 1
    MON = 2
    CMD = 3

    @staticmethod
    def toString(mode):
        """ Converts mode constant to string """
        return [ "INVALID", "OFF", "MON", "CMD" ][ mode ]

    @staticmethod
    def toPacketValue(mode):
        """ Converts mode constant to fri value """
        return [ FRI_STATE_INVALID, FRI_STATE_OFF,
                FRI_STATE_MON, FRI_STATE_CMD ][ mode ]


FRI_CTRL_POSITION = 1
FRI_CTRL_CART_IMP = 2
FRI_CTRL_JNT_IMP = 3
FRI_CTRL_OTHER = 0


FRI_QUALITY_INVALID =-1 
FRI_QUALITY_UNACCEPTABLE = 0
FRI_QUALITY_BAD = 1
FRI_QUALITY_OK = 2
FRI_QUALITY_PERFECT = 3

class FRIQuality(object):
    """ Enumeration of connection quality states """
    INVALID = 0
    BAD = 1
    UNACCEPTABLE = 2
    OK = 3
    PERFECT = 4

    @staticmethod
    def toString(quality):
        """ Converts quality constant to string """
        return [ "INVALID", "BAD", "UNACCEPTABLE", "OK", "PERFECT" ][ quality ]

    @staticmethod
    def toPacketValue(quality):
        """ Converts quality constant to fri value """
        return [ FRI_QUALITY_INVALID, FRI_QUALITY_BAD,
                FRI_QUALITY_UNACCEPTABLE, FRI_QUALITY_OK,
                FRI_QUALITY_PERFECT ][ quality ]


tFriHeader = """\
fri_uint16_t sendSeqCount
fri_uint16_t reflSeqCount
fri_uint16_t packetSize
fri_uint16_t datagramId
tFriHeader"""

tFriKrlData = """\
fri_float_t realData FRI_USER_SIZE
fri_int32_t intData FRI_USER_SIZE
fri_uint16_t boolData
fri_uint16_t fill
tFriKrlData"""

tFriIntfStatistics = """\
fri_float_t answerRate
fri_float_t latency
fri_float_t jitter
fri_float_t missRate
fri_uint32_t missCounter
tFriIntfStatistics"""
 
tFriIntfState = """\
fri_float_t timestamp
fri_uint16_t state
fri_uint16_t quality
fri_float_t desiredMsrSampleTime
fri_float_t desiredCmdSampleTime
fri_float_t safetyLimits
tFriIntfStatistics stat
tFriIntfState"""

tFriRobotState = """\
fri_uint16_t power
fri_uint16_t control
fri_uint16_t error
fri_uint16_t warning
fri_float_t temperature LBR_MNJ
tFriRobotState"""

tFriRobotData = """\
fri_float_t msrJntPos LBR_MNJ
fri_float_t msrCartPos FRI_CART_FRM_DIM
fri_float_t cmdJntPos LBR_MNJ
fri_float_t cmdJntPosFriOffset LBR_MNJ
fri_float_t cmdCartPos FRI_CART_FRM_DIM
fri_float_t cmdCartPosFriOffset FRI_CART_FRM_DIM
fri_float_t msrJntTrq LBR_MNJ
fri_float_t estExtJntTrq LBR_MNJ
fri_float_t estExtTcpFT FRI_CART_VEC
fri_float_t jacobian FRI_CART_VEC*LBR_MNJ
fri_float_t massMatrix LBR_MNJ*LBR_MNJ
fri_float_t gravity LBR_MNJ
tFriRobotData"""

tFriMsrData = """\
tFriHeader head
tFriKrlData krl
tFriIntfState intf
tFriRobotState robot
tFriRobotData data
tFriMsrData"""

tFriRobotCommand = """\
fri_uint32_t cmdFlags
fri_float_t jntPos LBR_MNJ
fri_float_t cartPos FRI_CART_FRM_DIM
fri_float_t addJntTrq LBR_MNJ
fri_float_t addTcpFT FRI_CART_VEC
fri_float_t jntStiffness LBR_MNJ
fri_float_t jntDamping LBR_MNJ
fri_float_t cartStiffness FRI_CART_VEC
fri_float_t cartDamping FRI_CART_VEC
tFriRobotCommand"""

tFriCmdData = """\
tFriHeader head
tFriKrlData krl
tFriRobotCommand cmd
tFriCmdData"""

tFriTest = """\
fri_uint16_t sendSeqCount
fri_uint16_t reflSeqCount
tFriHeader head
fri_uint16_t packetSize
tFriCmdData cmd
fri_uint16_t datagramId
tFriTest"""



