/*
 *   lwrmodels - Dynamic Robot Models of the KUKA Lightweight Robot IV
 *   Copyright (C) 2012 Vinzenz Bargsten <vb@bargsten.com>
 *
 *   This file is part of lwrmodels.
 *
 *   lwrmodels is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MODEL_H
#define MODEL_H

enum {
     NUMJ_MOD = 3
};

enum {
     NUMJ_ROB = 7
};

extern int SIMJOINTS[NUMJ_MOD];

void calc_residual(double resvec[2*NUMJ_MOD], const double q[NUMJ_MOD],
                   const double dq[NUMJ_MOD], const double dqp[NUMJ_MOD], 
                   const double ddqp[NUMJ_MOD], const double tau[NUMJ_MOD]);

void calc_jacobian(double resmat[2*NUMJ_MOD][2*NUMJ_MOD], const double cj, 
                   const double q[NUMJ_MOD], const double dq[NUMJ_MOD], 
                   const double dqp[NUMJ_MOD], const double ddqp[NUMJ_MOD]);

void calc_inertia_matrix(double resmat[NUMJ_MOD][NUMJ_MOD], 
                         const double q[NUMJ_MOD]);

void calc_gravity_torque_vector(double resvec[NUMJ_MOD], 
                                const double q[NUMJ_MOD]);

void calc_friction_torque_vector(double resvec[NUMJ_MOD],
                                 const double dq[NUMJ_MOD]);

void calc_coriolis_matrix(double resmat[NUMJ_MOD][NUMJ_MOD], 
                          const double q[NUMJ_MOD], const double dq[NUMJ_MOD]);

#endif
