/*
 *   lwrsundialsmodels - Adapts system equations of lwrmodels to be usable
 *                       with SUNDIALS IDA solver for implicit ODEs
 *                      
 *   Copyright (C) 2012 Vinzenz Bargsten <vb@bargsten.com>
 *
 *   This file is part of lwrsundialsmodels.
 *
 *   lwrsundialsmodels is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "sundialsmodel.h"
#include <stdio.h>
/*
 *--------------------------------------------------------------------
 * Functions called by IDA
 *--------------------------------------------------------------------
 */

/*
 * Define the system residual function. 
 */

int resrob(realtype tres, N_Vector yy, N_Vector yp, N_Vector rr, void * user_data)
{
  int i;
  realtype *yval, *ypval, *y;
  realtype q[NUMJ_MOD], dq[NUMJ_MOD], dqp[NUMJ_MOD], ddqp[NUMJ_MOD];
  TauData * data = (TauData *) user_data;

  
  yval = NV_DATA_S(yy); 
  ypval = NV_DATA_S(yp); 
  y = NV_DATA_S(rr);

  for (i=0;i<NUMJ_MOD;i++) {
    q[i] = yval[i*2];
    dq[i] = yval[i*2+1];
    dqp[i] = ypval[i*2];
    ddqp[i] = ypval[i*2+1];
  }
  
  calc_residual(y, q, dq, dqp, ddqp, data->tau);
  
  //printf("q1=%12.4le, q2=%12.4le\n", q[0], q[1]);
  return(0);
}


/*
 * Define the Jacobian function. 
 */

 int jacrob(int Neq, realtype tt,  realtype cj, 
            N_Vector yy, N_Vector yp, N_Vector resvec,
            DlsMat JJ, void *user_data,
            N_Vector tempv1, N_Vector tempv2, N_Vector tempv3)
 {
  int i, j;
  realtype *yval, *ypval, *col_i;
  realtype q[NUMJ_MOD], dq[NUMJ_MOD], dqp[NUMJ_MOD], ddqp[NUMJ_MOD], jac[2*NUMJ_MOD][2*NUMJ_MOD];
  
  
  yval = NV_DATA_S(yy); 
  ypval = NV_DATA_S(yp);
  for (i=0;i<NUMJ_MOD;i++) {
    q[i] = yval[i*2];
    dq[i] = yval[i*2+1];
    dqp[i] = ypval[i*2];
    ddqp[i] = ypval[i*2+1];
  }

    
    calc_jacobian(jac, cj, q, dq, dqp, ddqp);

    for (i=0; i<2*NUMJ_MOD; i++) {
        col_i = DENSE_COL(JJ, i);
        for (j=0; j<2*NUMJ_MOD; j++) {
             col_i[j] = jac[j][i];
        }
    }

    
/*  for(i=0;i<14;i++){
     col_i = DENSE_COL(JJ,i);
     printf("J(:,%i) = %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le %12.4le \n", i,
         col_i[0], col_i[1], col_i[2], col_i[3], col_i[4], col_i[5], col_i[6], col_i[7], col_i[8], col_i[9], col_i[10], col_i[11], col_i[12], col_i[13]);
  }*/
   return(0);
}

