#
#   python-lwrmodels - Python interface to the dynamic robot models of the
#                      lwrmodels C-library
#   Copyright (C) 2012 Vinzenz Bargsten <vb@bargsten.com>
#
#   This file is part of python-lwrmodels.
#
#   python-lwrmodels is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import numpy as np
from lwrmodels.lwr4_q2q4 import dynamics
from helper import read_float_array, print_matrix, print_vector

# get the number of joints described by the model
n = dynamics.numJointsModel

# Read user inputs
print("Enter {} joint positions (space separated): ".format(n), end="")
q = np.array(read_float_array(n))
print("Enter {} joint velocities (space separated): ".format(n), end="")
dq = np.array(read_float_array(n))
print("Enter {} joint actuation torques (space separated): ".format(n), end="")
tau = np.array(read_float_array(n))

# Compute Inertia Matrix B
B = dynamics.get_inertia_matrix(q)
# Compute the inverse of the Inertia Matrix
Binv = np.linalg.inv(B)

# Compute the Coriolis-/Centrifugal Matrix C
C = dynamics.get_coriolis_matrix(q, dq)

# Compute the gravity torque vector tau_g
tau_g = dynamics.get_gravity_torque_vector(q)

# Compute the friction torque vector tau_f
tau_f = dynamics.get_friction_torque_vector(dq)

# We can now compute the resulting joint accelerations ddq
ddq = Binv.dot( tau - C.dot(dq) - tau_g - tau_f )

# Print the results
print("\n\nInput set to:")
print("    q = {}^T rad".format(str(q)))
print("   dq = {}^T rad/s".format(str(dq)))
print("  tau = {}^T Nm".format(str(tau)))

print("\n\n\nOutput:")
print_matrix(B, "     B(q) = ")
print_matrix(C, "  C(q,dq) = ")
print_vector(tau_g, "     g(q) = ")
print_vector(tau_f, "tau_f(dq) = ")
print_matrix(Binv, "  B(q)^-1 = ")

print("Computed joint accelerations:");
print_vector(ddq, "ddq = ")

