#
#   python-lwrmodels - Python interface to the dynamic robot models of the
#                      lwrmodels C-library
#   Copyright (C) 2012 Vinzenz Bargsten <vb@bargsten.com>
#
#   This file is part of python-lwrmodels.
#
#   python-lwrmodels is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import numpy as np

def read_float_array(count=None):
    """Get an array of floating point values from user input
    
    Args:
        count (int): If not None, return a list of floats with exactly count
                     elements. If the user supplied to few values, the
                     remaining elements are set to zero. If the user supplied
                     more than count values, the superfluous values are
                     discarded. If None return all floats that could be read
                     from user input.
    
    Retruns:
        A list of floats.
    """
    
    userinput = input()
    userinput = userinput.replace("[","")
    userinput = userinput.replace("]","")
    userinput = userinput.replace(","," ")
    
    userinput = userinput.split(" ")
    
    vals = []
    for raw_val in userinput:
        if len(raw_val) < 1:
            continue
        
        try:
            vals += [float(raw_val)]
        except (ValueError):
            print("Could not interpret {}.".format(raw_val))
            continue
    
    if count is None or count < 1:
        count = len(vals)
    
    if len(vals) > count:
        print("You entered to many values, discarding rest.")
        del vals[count:]
        #while len(vals) > count:
            #vals.remove
    elif len(vals) < count:
        print("You entered to few values, filling with zeros.")
        vals += [0.0]* (count-len(vals))

    return vals

def print_vector(x, prefix=""):
    """Makes a 2-dimension array from x and then calls print_matrix.
    
    Args: 
        x (numpy.array): A numpy array with 1 dimension to be printed.
    
    Kwargs:
        prefix (string): Will be printed in front of vector, i.e. 'p = '.
    """
    
    A = np.zeros((x.shape[0], 1))
    A[:,0] = x[:]
    print_matrix(A, prefix)
    
  
def print_matrix(A, prefix=""):
    """Prints a matrix to standard output in human readable format.
    
    Args: 
        A (numpy.array): A numpy array with 2 dimensions to be printed.
    
    Kwargs:
        prefix (string): Will be printed in front of matrix, i.e. 'A = '.
    """
    
    n = A.shape[0]
    
    m = 1
    if len(A.shape) == 2:
        m = A.shape[1]
    
    prefix_space = "".join([" "]*len(prefix))
    fmtstr = "{}| " + " ".join(["{:10.3e}"]*m) + " |"
    
    midlen = 10 * m + 2 + (m-1) -2
    topbtmline = prefix_space + "--" + "".join([" "]*midlen) + "--"
    
    print(topbtmline)
    
    for row in range(n):
        curprefix = prefix_space
        if n % 2 == 0 and row == (n/2):
            print(prefix + "| " + "".join([" "]*midlen) + " |")
        elif  n % 2 == 1 and row == ((n-1)/2):
            curprefix = prefix
            
        print(fmtstr.format(curprefix, *A[row,:]))
       
    print(topbtmline)
    
    