import sys
import os
from glob import glob

import numpy as np

from distutils.core import setup
import distutils
from distutils.extension import Extension
from distutils.version import LooseVersion

hascy = False
try:
    from Cython.Distutils import build_ext
    hascy = True
except ImportError:
    pass


prefix = ""
for arg in sys.argv:
    if arg.startswith("--prefix="):
        prefix = arg.split("=",1)[1]
        break

    if arg == "--user":
        prefix = os.path.expanduser("~")
        break

    if arg.startswith("--home="):
        prefix = arg.split("=",1)[1]
        prefix = os.path.expanduser(prefix)
        break

print(prefix)
#
    
#sys.setrecursionlimit(10*sys.getrecursionlimit())


packname = "lwrmodels"
modname = "dynamics"
modext = ".c"
robnames = ('lwr4', 'lwr4_q1q2q4', 'lwr4_q2q4')


inc_searchpaths = ["/usr/include"]
inc_searchpaths += ["/usr/local/include"]

lib_searchpaths = glob("/usr/lib*")
lib_searchpaths += glob("/usr/local/lib*")

if prefix != "":
    inc_searchpaths += glob(os.path.join(prefix, "include"))
    lib_searchpaths += glob(os.path.join(prefix, "lib*"))

lwrmodels_dirname = "lwrmodels*"
lwrmodels_includedir = None
lwrmodels_includedir_isspec = False
lwrmodels_libdir = None
lwrmodels_libdir_isspec = False
try:
    lwrmodels_includedir = os.environ["LWRMODELS_INCLUDE"]
    if not os.path.exists(os.path.join(lwrmodels_includedir, robnames[0])):
        print(("Warning: Specified lwrmodels include directory "
               "(LWRMODELS_INCLUDE) seems incorrect, because no {} "
               "subdirectory found. Discarding...").format(robnames[0]))
    else:
        lwrmodels_includedir_isspec = True
except KeyError:
    pass

try:
    lwrmodels_libdir = os.environ["LWRMODELS_LIB"]
    if not os.path.exists(os.path.join(lwrmodels_libdir, robnames[0])):
        print(("Warning: Specified lwrmodels library directory "
               "(LWRMODELS_LIB) seems incorrect, because no {} "
               "subdirectory found. Discarding...").format(robnames[0]))
    else:
        lwrmodels_libdir_isspec = True
except KeyError:
    pass
        

if not lwrmodels_includedir_isspec:
    if lwrmodels_libdir_isspec:
        print(("Warning: You should set both environment variables, "
               "LWRMODELS_LIB and LWRMODELS_INCLUDE, or none. Guessing now..."))
        tmp = os.path.split(lwrmodels_libdir)
        if len(tmp[1]) < 1:
            tmp = os.path.split(tmp[0])
        if len(tmp[1]) > 0:
            lwrmodels_dirname = tmp[1]
            inc_searchpaths += glob(os.path.join(lwrmodels_libdir, os.pardir, 
                os.pardir, "include"))
    
    for p in reversed(inc_searchpaths):
        lwrmodels_includedirs = glob(os.path.join(p, lwrmodels_dirname))
        if len(lwrmodels_includedirs) < 1:
            continue
        else:
            lwrmodels_includedirs.sort(key=LooseVersion)
            lwrmodels_includedir = lwrmodels_includedirs[-1]
            break
            
if lwrmodels_includedir == None:
    print("Warning: No lwrmodels include directory found.")
    print("I have searched for {} in these paths:".format(lwrmodels_dirname))
    print(list(reversed(inc_searchpaths)))
    lwrmodels_includedir = "/usr/local/include/lwrmodels0.8"
else:
    print("Note: Using lwrmodels include directory '{}'."\
            .format(lwrmodels_includedir))
    tmp = os.path.split(lwrmodels_includedir)
    if len(tmp[1]) < 1:
        tmp = os.path.split(tmp[0])
    if len(tmp[1]) > 0:
        lwrmodels_dirname = tmp[1]
        lib_searchpaths += glob(os.path.join(lwrmodels_includedir, os.pardir, 
            os.pardir, "lib*"))


if not lwrmodels_libdir_isspec:
    if lwrmodels_includedir_isspec:
        print(("Warning: You should set both environment variables,"
               "LWRMODELS_LIB and LWRMODELS_INCLUDE, or none."))
    for p in reversed(lib_searchpaths):
        lwrmodels_libdirs = glob(os.path.join(p, lwrmodels_dirname))
        if len(lwrmodels_libdirs) < 1:
            continue
        else:
            lwrmodels_libdirs.sort(key=LooseVersion)
            lwrmodels_libdir = lwrmodels_libdirs[-1]
            break
            
if lwrmodels_libdir == None:
    print("Warning: No lwrmodels library directory found.")
    print("I have searched for {} in these paths:".format(lwrmodels_dirname))
    print(list(reversed(lib_searchpaths)))
    lwrmodels_libdir = "/usr/local/lib/lwrmodels0.8"
else:
    print("Note: Using lwrmodels library directory '{}'."\
            .format(lwrmodels_libdir))


usecy = True
needcy = False
for rob in robnames:
    if not os.path.exists(os.path.join(packname, rob, modname+".c")):
        needcy = True
        break

if needcy and not hascy:
    print(("C source files not existing -- They need to be generated "
           "from pyx/pxd files by Cython, but Cython cannot be imported.\n"
           "Check that Cython is installed and in the PYTHONPATH.\n"))
    sys.exit(1)
elif not needcy and not hascy:
    print(("Existing C source files will be used, "
           "because Cython cannot be imported.\n"))
    usecy = False

if usecy:
    modext = ".pyx"

extmods = []
packs = [packname]

for rob in robnames:
    ext_name = ".".join([packname, rob, modname])
    ext_dir = [os.path.join(packname, rob, modname+modext)]

    inc_dirs = ['.', packname]
    inc_dirs += [np.get_include()]
    inc_dirs += [os.path.join(lwrmodels_includedir, rob)]

    lib_dirs = ['.']
    lib_dirs += [os.path.join(lwrmodels_libdir, rob)]

    libs = ["m","model"]

    extmods += [
                Extension(
                    ext_name, ext_dir,
                    include_dirs=inc_dirs,
                    library_dirs=lib_dirs,
                    libraries=libs
                    )
                ]
    packs += [packname + "." + rob]

setup_args = {}
setup_args['name'] = 'python-lwrmodels'
setup_args['version'] = '0.8'
setup_args['description'] = ''
setup_args['long_description'] = open('README').read()
setup_args['license'] = 'GPLv3'
setup_args['author'] = 'Vinzenz Bargsten'
setup_args['author_email'] = 'vb@bargsten.com'
setup_args['url'] = 'http://vb.bargsten.com/robotics/python-lwrmodels'
#setup_args['download_url'] = ''
setup_args['package_data'] = {'lwrmodels': [
                                    '*.pxd','*/*.pxd',
                                    '*.pyx','*/*.pyx',
                                    'README','INSTALL'
                                    ]
                                }
# additional files needed by the module distribution
#setup_args['data_files'] = [ ('bitmaps', ['bm/b1.gif', 'bm/b2.gif'])],
setup_args['ext_modules'] = extmods
setup_args['packages'] = packs

if usecy:
    setup(
        cmdclass={'build_ext':build_ext},
        **setup_args
    )
else:
    setup(
        **setup_args
    )