#
#   python-lwrsim - Simulates the robot motion dynamics using
#                   the dynamic robot models of the lwrmodels and
#                   lwrsundialsmodel library with SUNDIALS IDA.
#   Copyright (C) 2012 Vinzenz Bargsten <vb@bargsten.com>
#
#   This file is part of python-lwrsim.
#
#   python-lwrsim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#   This file incorporates work covered by the following copyright and  
#   permission notice:  
#
#    Copyright (c) 2010 Runar Tenfjord. All rights reserved.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions are
#    met:
#
#    1. Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#    
#    2. Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in
#        the documentation and/or other materials provided with the
#        distribution.
#
#    3. Neither the name of Tenko nor the names of its contributors may
#        be used to endorse or promote products derived from this software
#        without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INFRAE OR
#    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


from sundials.sundials cimport N_Vector_Serial, IDASettings
from sundials.SundialsLib cimport realtype, N_Vector, DlsMat
#from cymodel cimport NUMJ_MOD

cdef extern from "sundialsmodel.h":
    enum: NUMJ_MOD
    enum: NUMJ_ROB
    int SIMJOINTS[]

    ctypedef struct TauData "TauData":
        realtype tau[NUMJ_MOD]
        
    int resrob(realtype tres, N_Vector yy, N_Vector yp, 
               N_Vector resval, void * user_data)

    int jacrob(int Neq, realtype tt,  double c, 
               N_Vector yy, N_Vector yp, N_Vector resvec,
               DlsMat JJ, void *user_data,
               N_Vector tempv1, N_Vector tempv2, N_Vector tempv3)

   
    
cdef class DynamicsSimulator:
    cdef void *thisptr
    cdef public double sampleTime
    #cdef public int numJoints_model
    #cdef public int numJoints_robot
    #cdef public list simulatedJoints
    
    cdef double t
    
    cdef N_Vector_Serial y
    cdef N_Vector_Serial ydot

    cdef public IDASettings settings
    cdef TauData userdata
    cdef realtype tau[NUMJ_MOD]
    cpdef calc_next_state(self, list torques)
    cpdef set_state(self, list q, list dq, makeconsistent)
    