#
#   python-lwrsim - Simulates the robot motion dynamics using
#                   the dynamic robot models of the lwrmodels and
#                   lwrsundialsmodel library with SUNDIALS IDA.
#   Copyright (C) 2012 Vinzenz Bargsten <vb@bargsten.com>
#
#   This file is part of python-lwrsim.
#
#   python-lwrsim is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


'''
Created on 15.11.2011

@author: vb
'''

import os.path
from lbrsim import LBRSimulator
import numpy
from dynsim import DynamicsSimulator as DynSim
import time
import sys, code

sys.path.append(os.path.abspath("../../lbrmodel/src")) 
from cymodel import get_gravity_torque_vector as calc_G

if sys.version > '3':
    PY3 = True
else:
    PY3 = False

from math import pi

#########################
### SETUP ROBOT MODEL ###
#########################


# Sampling Time
Ts=0.01




##############################
### SETUP SIMULATION MODEL ###
##############################


q0 = [0,00.0/180*pi,0, 90.0/180*pi,0,0,0]
dq0 = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]

robotModel = DynSim(sampleTime=Ts, q0=q0, dq0=dq0, mxsteps=50000, abstol=1.0e-6, reltol=1.0e-5)
#
frisim = LBRSimulator(robotModel, gravityFunct=calc_G, debug=True)
#frisim.gravityFunct = calc_G
frisim.start()
#frisim._run()
print("frisim pid ", frisim.process.pid)

while not frisim.isRunning.value:
    time.sleep(1.0)

while frisim.isRunning.value:
    time.sleep(1.0)
    


frisim.stop()
frisim.process.join()

print("END")