

#include_dirs=['.',packname, sundials_includedir,np.get_include(),lwrmodels_includedir,lwrsdmodels_includedir+"/"+rob],
#library_dirs=[".",lwrmodels_libdir+"/"+rob,lwrsdmodels_libdir+"/"+rob,sundials_libdir],
#libraries=["m","model","sundialsmodel","sundials_nvecserial", "sundials_ida",'lapack'])]
    
import sys
import os

import numpy as np

from distutils.core import setup
from distutils.extension import Extension
from distutils.version import LooseVersion
from glob import glob

hascy = False
try:
    from Cython.Distutils import build_ext
    hascy = True
except ImportError:
    pass

prefix = ""
for arg in sys.argv:
    if arg.startswith("--prefix="):
        prefix = arg.split("=",1)[1]
        break

    if arg == "--user":
        prefix = os.path.expanduser("~")
        break

    if arg.startswith("--home="):
        prefix = arg.split("=",1)[1]
        prefix = os.path.expanduser(prefix)
        break

        
packname = "lwrsim"
modname = "dynsim"
modext = ".c"
robnames = ('lwr4', 'lwr4_q1q2q4', 'lwr4_q2q4')


inc_searchpaths = ["/usr/include"]
inc_searchpaths += ["/usr/local/include"]

lib_searchpaths = glob("/usr/lib*")
lib_searchpaths += glob("/usr/local/lib*")

if prefix != "":
    inc_searchpaths += glob(os.path.join(prefix, "include"))
    lib_searchpaths += glob(os.path.join(prefix, "lib*"))
    
python_sundials_inc_isspec = False
try:
    python_sundials_inc = os.environ["PYTHON_SUNDIALS_INCLUDE"]
    if not os.path.exists(os.path.join(python_sundials_inc,
        "sundials", "sundials.pxd")):
            
        print(("Warning: Specified python-sundials include directory "
               "(PYTHON_SUNDIALS_INCLUDE) seems incorrect, because no file "
               "sundials/sundials.pxd found. Discarding..."))
    else:
        python_sundials_inc_isspec = True
except KeyError:
    pass

if not python_sundials_inc_isspec:
    try:
        from sundials import get_pxd_include
        python_sundials_inc = os.path.join(get_pxd_include(), os.pardir)
    except (ImportError):
        print(("Warning: Could not find python-sundials pxd (include) files.\n"
               "Set the PYHTON_SUNDIALS_INCLUDE environment variable."))
        python_sundials_inc = "../python-sundials/sundials"


lwrmodels_dirname = "lwrmodels*"
lwrmodels_includedir = None
lwrmodels_includedir_isspec = False
lwrmodels_libdir = None
lwrmodels_libdir_isspec = False
try:
    lwrmodels_includedir = os.environ["LWRMODELS_INCLUDE"]
    if not os.path.exists(os.path.join(lwrmodels_includedir, robnames[0])):
        print(("Warning: Specified lwrmodels include directory "
               "(LWRMODELS_INCLUDE) seems incorrect, because no {} "
               "subdirectory found. Discarding...").format(robnames[0]))
    else:
        lwrmodels_includedir_isspec = True
except KeyError:
    pass

try:
    lwrmodels_libdir = os.environ["LWRMODELS_LIB"]
    if not os.path.exists(os.path.join(lwrmodels_libdir, robnames[0])):
        print(("Warning: Specified lwrmodels library directory "
               "(LWRMODELS_LIB) seems incorrect, because no {} "
               "subdirectory found. Discarding...").format(robnames[0]))
    else:
        lwrmodels_libdir_isspec = True
except KeyError:
    pass
        

if not lwrmodels_includedir_isspec:
    if lwrmodels_libdir_isspec:
        print(("Warning: You should set both environment variables, "
               "LWRMODELS_LIB and LWRMODELS_INCLUDE, or none. Guessing now..."))
        tmp = os.path.split(lwrmodels_libdir)
        if len(tmp[1]) < 1:
            tmp = os.path.split(tmp[0])
        if len(tmp[1]) > 0:
            lwrmodels_dirname = tmp[1]
            inc_searchpaths += glob(os.path.join(lwrmodels_libdir, os.pardir, 
                os.pardir, "include"))
    
    for p in reversed(inc_searchpaths):
        lwrmodels_includedirs = glob(os.path.join(p, lwrmodels_dirname))
        if len(lwrmodels_includedirs) < 1:
            continue
        else:
            lwrmodels_includedirs.sort(key=LooseVersion)
            lwrmodels_includedir = lwrmodels_includedirs[-1]
            break
            
if lwrmodels_includedir == None:
    print("Warning: No lwrmodels include directory found.")
    print("I have searched for {} in these paths:".format(lwrmodels_dirname))
    print(list(reversed(inc_searchpaths)))
    lwrmodels_includedir = "/usr/local/include/lwrmodels0.8"
else:
    print("Note: Using lwrmodels include directory '{}'."\
            .format(lwrmodels_includedir))
    tmp = os.path.split(lwrmodels_includedir)
    if len(tmp[1]) < 1:
        tmp = os.path.split(tmp[0])
    if len(tmp[1]) > 0:
        lwrmodels_dirname = tmp[1]
        lib_searchpaths += glob(os.path.join(lwrmodels_includedir, os.pardir, 
            os.pardir, "lib*"))


if not lwrmodels_libdir_isspec:
    if lwrmodels_includedir_isspec:
        print(("Warning: You should set both environment variables,"
               "LWRMODELS_LIB and LWRMODELS_INCLUDE, or none."))
    for p in reversed(lib_searchpaths):
        lwrmodels_libdirs = glob(os.path.join(p, lwrmodels_dirname))
        if len(lwrmodels_libdirs) < 1:
            continue
        else:
            lwrmodels_libdirs.sort(key=LooseVersion)
            lwrmodels_libdir = lwrmodels_libdirs[-1]
            break
            
if lwrmodels_libdir == None:
    print("Warning: No lwrmodels library directory found.")
    print("I have searched for {} in these paths:".format(lwrmodels_dirname))
    print(list(reversed(lib_searchpaths)))
    lwrmodels_libdir = "/usr/local/lib/lwrmodels0.8"
else:
    print("Note: Using lwrmodels library directory '{}'."\
            .format(lwrmodels_libdir))


# Find lwrsundialsmodels include and library directory
lwrsundialsmodels_dirname = "lwrsundialsmodels*"
lwrsundialsmodels_includedir = None
lwrsundialsmodels_includedir_isspec = False
lwrsundialsmodels_libdir = None
lwrsundialsmodels_libdir_isspec = False
try:
    lwrsundialsmodels_includedir = os.environ["LWRSUNDIALSMODELS_INCLUDE"]
    if not os.path.exists(
        os.path.join(lwrsundialsmodels_includedir, robnames[0])):
            
        print(("Warning: Specified lwrsundialsmodels include directory "
               "(LWRSUNDIALSMODELS_INCLUDE) seems incorrect, because no {} "
               "subdirectory found. Discarding...").format(robnames[0]))
    else:
        lwrsundialsmodels_includedir_isspec = True
except KeyError:
    pass

try:
    lwrsundialsmodels_libdir = os.environ["LWRSUNDIALSMODELS_LIB"]
    if not os.path.exists(os.path.join(lwrsundialsmodels_libdir, robnames[0])):
        print(("Warning: Specified lwrsundialsmodels library directory "
               "(LWRSUNDIALSMODELS_LIB) seems incorrect, because no {} "
               "subdirectory found. Discarding...").format(robnames[0]))
    else:
        lwrsundialsmodels_libdir_isspec = True
except KeyError:
    pass
        

if not lwrsundialsmodels_includedir_isspec:
    if lwrsundialsmodels_libdir_isspec:
        print(("Warning: You should set both environment variables, "
               "LWRSUNDIALSMODELS_LIB and LWRSUNDIALSMODELS_INCLUDE, "
               "or none. Guessing now..."))
        tmp = os.path.split(lwrsundialsmodels_libdir)
        if len(tmp[1]) < 1:
            tmp = os.path.split(tmp[0])
        if len(tmp[1]) > 0:
            lwrsundialsmodels_dirname = tmp[1]
            inc_searchpaths += glob(os.path.join(lwrsundialsmodels_libdir,
                os.pardir, os.pardir, "include"))
    
    for p in reversed(inc_searchpaths):
        lwrsundialsmodels_includedirs = glob(
            os.path.join(p, lwrsundialsmodels_dirname))
        if len(lwrsundialsmodels_includedirs) < 1:
            continue
        else:
            lwrsundialsmodels_includedirs.sort(key=LooseVersion)
            lwrsundialsmodels_includedir = lwrsundialsmodels_includedirs[-1]
            break
            
if lwrsundialsmodels_includedir == None:
    print("Warning: No lwrsundialsmodels include directory found.")
    print("I have searched for {} in these paths:".format(
        lwrsundialsmodels_dirname))
    print(list(reversed(inc_searchpaths)))
    lwrsundialsmodels_includedir = "/usr/local/include/lwrsundialsmodels0.8"
else:
    print("Note: Using lwrsundialsmodels include directory '{}'."\
            .format(lwrsundialsmodels_includedir))
    tmp = os.path.split(lwrsundialsmodels_includedir)
    if len(tmp[1]) < 1:
        tmp = os.path.split(tmp[0])
    if len(tmp[1]) > 0:
        lwrsundialsmodels_dirname = tmp[1]
        lib_searchpaths += glob(os.path.join(lwrsundialsmodels_includedir,
            os.pardir, os.pardir, "lib*"))


if not lwrsundialsmodels_libdir_isspec:
    if lwrsundialsmodels_includedir_isspec:
        print(("Warning: You should set both environment variables,"
               "LWRSUNDIALSMODELS_LIB and LWRSUNDIALSMODELS_INCLUDE, or none."))
    for p in reversed(lib_searchpaths):
        lwrsundialsmodels_libdirs = glob(
            os.path.join(p, lwrsundialsmodels_dirname))
        if len(lwrsundialsmodels_libdirs) < 1:
            continue
        else:
            lwrsundialsmodels_libdirs.sort(key=LooseVersion)
            lwrsundialsmodels_libdir = lwrsundialsmodels_libdirs[-1]
            break
            
if lwrsundialsmodels_libdir == None:
    print("Warning: No lwrsundialsmodels library directory found.")
    print("I have searched for {} in these paths:".format(
        lwrsundialsmodels_dirname))
    print(list(reversed(lib_searchpaths)))
    lwrsundialsmodels_libdir = "/usr/local/lib/lwrsundialsmodels0.8"
else:
    print("Note: Using lwrsundialsmodels library directory '{}'."\
            .format(lwrsundialsmodels_libdir))

# Find sundials include and library directory
sundials_dirname = "sundials"
sundials_includedir = None
sundials_includedir_isspec = False
sundials_libdir = None
sundials_libdir_isspec = False
try:
    sundials_includedir = os.environ["SUNDIALS_INCLUDE"]
    if not os.path.exists(
        os.path.join(sundials_includedir, "sundials", "sundials_dense.h")):
            
        print(("Warning: Specified sundials include directory "
               "(SUNDIALS_INCLUDE) seems incorrect, because no "
               "sundials/sundials_dense.h file found. Discarding..."))
    else:
        sundials_includedir_isspec = True
except KeyError:
    pass

try:
    sundials_libdir = os.environ["SUNDIALS_LIB"]
    if len(glob(os.path.join(sundials_libdir, "libsundials_ida.so*")))<1:
        print(("Warning: Specified sundials library directory "
               "(SUNDIALS_LIB) seems incorrect, because no "
               "libsundials_ida.so* file found. Discarding..."))
    else:
        sundials_libdir_isspec = True
except KeyError:
    pass
        

if not sundials_includedir_isspec:
    if sundials_libdir_isspec:
        print(("Warning: You should set both environment variables, "
               "SUNDIALS_LIB and SUNDIALS_INCLUDE, "
               "or none. Guessing now..."))

        inc_searchpaths += os.glob(os.path.join(sundials_libdir,
                os.pardir, "include"))
    
    for p in reversed(inc_searchpaths):
        sundials_includedirs = glob(
            os.path.join(p, sundials_dirname))
        if len(sundials_includedirs) < 1:
            continue
        else:
            sundials_includedirs.sort(key=LooseVersion)
            sundials_includedir = os.path.join(sundials_includedirs[-1],
                os.pardir)
            break
            
if sundials_includedir == None:
    print("Warning: No sundials include directory found.")
    print("I have searched for {} in these paths:".format(
        sundials_dirname))
    print(list(reversed(inc_searchpaths)))
    sundials_includedir = "/usr/local/include"
else:
    print("Note: Using sundials include directory '{}'."\
            .format(sundials_includedir))
            
    lib_searchpaths += glob(os.path.join(sundials_includedir,
        os.pardir, "lib*"))


if not sundials_libdir_isspec:
    if sundials_includedir_isspec:
        print(("Warning: You should set both environment variables,"
               "SUNDIALS_LIB and SUNDIALS_INCLUDE, or none."))
    for p in reversed(lib_searchpaths):
        sundials_libdirs = glob(
            os.path.join(p, "libsundials_ida.so*"))
        if len(sundials_libdirs) < 1:
            continue
        else:
            sundials_libdirs.sort(key=LooseVersion)
            sundials_libdir = os.path.split(sundials_libdirs[-1])[0]
            break
            
if sundials_libdir == None:
    print("Warning: No sundials library directory found.")
    print("I have searched for {} in these paths:".format(
        "libsundials_ida.so*"))
    print(list(reversed(lib_searchpaths)))
    sundials_libdir = "/usr/local/lib"
else:
    print("Note: Using sundials library directory '{}'."\
            .format(sundials_libdir))


usecy = True
needcy = False
for rob in robnames:
    if not os.path.exists(os.path.join(packname, rob, modname+".c")):
        needcy = True
        break

if needcy and not hascy:
    print(("C source files not existing -- They need to be generated "
           "from pyx/pxd files by Cython, but Cython cannot be imported.\n"
           "Check that Cython is installed and in the PYTHONPATH.\n"))
    sys.exit(1)
elif not needcy and not hascy:
    print(("Existing C source files will be used, "
           "because Cython cannot be imported.\n"))
    usecy = False

if usecy:
    modext = ".pyx"

extmods = []
packs = [packname]

for rob in robnames:
    ext_name = ".".join([packname, rob, modname])
    ext_dir = [os.path.join(packname, rob, modname+modext)]
    
    inc_dirs = ['.', packname]
    inc_dirs += [sundials_includedir]
    inc_dirs += [python_sundials_inc]
    inc_dirs += [np.get_include()]
    inc_dirs += [os.path.join(lwrmodels_includedir, rob)]
    inc_dirs += [os.path.join(lwrsundialsmodels_includedir, rob)]

    lib_dirs = ['.']
    lib_dirs += [os.path.join(lwrmodels_libdir, rob)]
    lib_dirs += [os.path.join(lwrsundialsmodels_libdir, rob)]
    lib_dirs += [sundials_libdir]

    libs = ["m","model","sundialsmodel"]
    libs += ["sundials_nvecserial", "sundials_ida",'lapack']
    
    extmods += [
                Extension(
                    ext_name, ext_dir,
                    include_dirs=inc_dirs,
                    library_dirs=lib_dirs,
                    libraries=libs
                    )
                ]
    packs += [packname + "." + rob]

setup_args = {}
setup_args['name'] = 'python-lwrsim'
setup_args['version'] = '0.8'
setup_args['description'] = ''
setup_args['long_description'] = open('README').read()
setup_args['license'] = 'GPLv3'
setup_args['author'] = 'Vinzenz Bargsten'
setup_args['author_email'] = 'vb@bargsten.com'
setup_args['url'] = 'http://vb.bargsten.com/robotics/python-lwrsim'
#setup_args['download_url'] = ''
setup_args['package_data'] = {'lwrsim': [
                                    '*.pxd','*/*.pxd',
                                    '*.pyx','*/*.pyx',
                                    'README','INSTALL'
                                    ]
                                }
# additional files needed by the module distribution
#setup_args['data_files'] = [ ('bitmaps', ['bm/b1.gif', 'bm/b2.gif'])],
setup_args['ext_modules'] = extmods
setup_args['packages'] = packs

if usecy:
    setup(
        cmdclass={'build_ext':build_ext},
        **setup_args
    )
else:
    setup(
        **setup_args
    )