#!/usr/bin/python
# -*- utf8 -*-
"""
Copyright (C) 2012 Vinzenz Bargsten <vb@bargsten.com>
Copyright (C) 2010 Runar Tenfjord <runar.tenfjord@gmail.com>
"""
import sys
import os
import glob
import shutil

from distutils.core import setup
from distutils.extension import Extension

hascy = False
try:
    from Cython.Distutils import build_ext
    hascy = True
except ImportError:
    pass

usecy = True
needcy = False
if not os.path.exists(os.path.join("sundials", "sundials.c")):
    needcy = True

if needcy and not hascy:
    print(("C source files not existing -- They need to be generated "
           "from pyx/pxd files by Cython, but Cython cannot be imported.\n"
           "Check that Cython is installed and in the PYTHONPATH.\n"))
    sys.exit(1)
elif not needcy and not hascy:
    print(("Existing C source files will be used, "
           "because Cython cannot be imported.\n"))
    usecy = False

if usecy:
    modext = ".pyx"
else:
    modext = ".c"
    
    
classifiers = '''\
Environment :: Console
Development Status :: 4 - Beta
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Operating System :: OS Independent
Programming Language :: Python
Topic :: Scientific/Engineering :: Mathematics
'''

setup_args = {}
setup_args['name'] = 'python-sundials'
setup_args['version'] = '0.5-vb'
setup_args['description'] = 'Sundials solver library wrapper'
setup_args['long_description'] = open('README.txt').read()
setup_args['classifiers'] = [value for value in classifiers.split("\n") \
                                if value]
setup_args['author'] = 'Runar Tenfjord'
setup_args['author_email'] = 'runar.tenfjord@google.com'
setup_args['license'] = 'BSD'
setup_args['download_url'] = 'http://pypi.python.org/pypi/python-sundials/'
setup_args['url'] = 'http://code.google.com/p/python-sundials/'
setup_args['platforms'] = ['any']
setup_args['ext_modules'] = [   
    Extension("sundials.sundials", 
              sources = ["sundials/sundials"+modext,],
              depends = glob.glob('sundials/*.pxi'),
              include_dirs = ['sundials', 'sundials/include'],
              library_dirs=['sundials'],
              libraries = ['sundials_cvode','sundials_ida',
                           'sundials_kinsol', 'sundials_nvecserial','lapack'],
              )
    ]

setup_args['packages'] = ['sundials']
setup_args['package_dir'] = {'sundials' : 'sundials'}
setup_args['package_data'] = {'sundials':['*.pxd', '*.pyx', '*.pxi']}


if usecy:
    setup(
        cmdclass={'build_ext':build_ext},
        **setup_args
    )
else:
    setup(
        **setup_args
    )